(function (global) {
    "use strict";

    global.FRCC = global.FRCC || {};
    global.FRCC.Portal = global.FRCC.Portal || {};
    global.FRCC.Portal.Validations = (function() {

        class PreValidationResults {
            constructor() {
                this._results = {};
            }

            add = (identifier, result) => this._results[identifier] = result;
            get = (identifier) => this._results[identifier];
        }

        var setupValidations = function(validationConfiguration) {

            _setupPreSubmitValidation(validationConfiguration);

            validationConfiguration.Validations.forEach((validation, index) => {
                switch(validation.ActionType)
                {
                    case 203300000: // Field Input
                        _setupFieldValidation(validation)
                        break;
                    case 203300002: // Table Entry
                        _setupTableValidation(validation)
                        break;
					// File upload validation (203300001) will be handled in documents.js web file
                }
            });
        };

        var _setupPreSubmitValidation = function (validationConfiguration) {
            $(".submit-btn").each((index, button) => {
                const submitFunction = button.onclick;
                const stepNumber = validationConfiguration.StepNumbers[index];
                button.onclick = (event) => {
                    let preventSubmitFlag = false;
                    const preventSubmit = () => preventSubmitFlag = true;

                    validationConfiguration.Validations
                        .filter(v => v.StepNumber == stepNumber)
                        // Run asynchronous prevalidations synchronously
                        .reduce((previousPromise, v) => preventSubmitFlag
                            ? previousPromise
                            : previousPromise.then(() => _handlePreSubmitValidation(v, preventSubmit)),
                            Promise.resolve())
                        .then(() => {
                            if (!preventSubmitFlag) {
                                submitFunction(event);
                            }
                        })
                        .catch((error) => {
                            console.log(`An error occurred during prevalidation: ${error}.`);
                            alert(`An error occurred during prevalidation: ${error}.`);
                            preventSubmit();
                        });
                }
            });
        }

        var _handlePreSubmitValidation = function (validation, preventSubmit) {
            switch (validation.FieldValidationType) {
                case 643_260_006: // Valid Address
                    FRCC.Portal.Utilities.showProgressIndicator("Validating address...");

                    var eygaApiContext = _getEygaApiContextFromForm();
                    $(`input#${validation.Identifier}`).data('folder');

                    var request = {
                        "AddressLines": validation.AddressLineFields.map(f => $(`input#${f.FieldLogicalName}`)?.[0]?.value),
                        "Locality": $(`input#${validation.LocalityField.FieldLogicalName}`)?.[0]?.value,
                        "AdministrativeArea": $(`input#${validation.AdministrativeAreaField.FieldLogicalName}`)?.[0]?.value,
                        "PostalCode": $(`input#${validation.PostalCodeField.FieldLogicalName}`)?.[0]?.value,
                        "RegionCode": validation.RegionCodeField
                            ? ($(`input#${validation.RegionCodeField.FieldLogicalName}`)?.[0]?.value
                                || validation.RegionCodeDefault)
                            : validation.RegionCodeDefault
                    };
                    if(!request.AddressLines[0] && !request.AddressLines[1] && !request.Locality && !request.AdministrativeArea && !request.PostalCode )
                    {
                        FRCC.Portal.Validations.PreValidationResults.add(validation.Identifier, true);
                        return;
                    }

                    return FRCC.Portal.Utilities.callEygaApi(
                            FRCC.Portal.Utilities.ApiCall.ValidateAddress,
                            eygaApiContext,
                            request)
                        .then((response) => {
                            if (response.ValidationLevel == "Invalid" || !response.Address) {
                                FRCC.Portal.Validations.PreValidationResults.add(validation.Identifier, false);
                                return;
                            }
                            else if (request.AddressLines.every((x, i) => x?.trim()?.toLowerCase()
                                    == response.Address.AddressLines?.[i]?.trim()?.toLowerCase())
                                && request.Locality?.trim()?.toLowerCase() == response.Address.Locality?.trim()?.toLowerCase()
                                && request.AdministrativeArea?.trim()?.toLowerCase() == response.Address.AdministrativeArea?.trim()?.toLowerCase()
                                && request.PostalCode?.trim()?.toLowerCase() == response.Address.PostalCode?.trim()?.toLowerCase()
                                && request.RegionCode?.trim()?.toLowerCase() == response.Address.RegionCode?.trim()?.toLowerCase())
                            {
                                FRCC.Portal.Validations.PreValidationResults.add(validation.Identifier, true);
                                return;
                            }

                            FRCC.Portal.Utilities.hideProgressIndicator();
                            return _openAddressConfirmationDialog(validation, request, response)
                                .then(function accept(address) {
                                    if (address) {
                                        validation.AddressLineFields.forEach((f, i) =>
                                            $(`input#${f.FieldLogicalName}`).val(address.AddressLines[i])
                                        );
                                        $(`input#${validation.LocalityField.FieldLogicalName}`).val(address.Locality);
                                        $(`input#${validation.AdministrativeAreaField.FieldLogicalName}`).val(address.AdministrativeArea);
                                        $(`input#${validation.PostalCodeField.FieldLogicalName}`).val(address.PostalCode);
                                        if (validation.RegionCodeField) {
                                            $(`input#${validation.RegionCodeField.FieldLogicalName}`).val(address.RegionCode);
                                        }
                                    }

                                    _clearValidationErrors(validation);

                                    FRCC.Portal.Validations.PreValidationResults.add(validation.Identifier, true);
                                },
                                function cancel() {
                                    preventSubmit();
                                });
                        })
                        .finally(() => {
                            FRCC.Portal.Utilities.hideProgressIndicator();
                        });
                        
                default:
                    return Promise.resolve();
            }
        }

        var _clearValidationErrors = function (validation) {
            validation.AddressLineFields.forEach((f, i) =>
                $(`input#${f.FieldLogicalName}`).parent().removeClass('has-error')
            );
            $(`input#${validation.LocalityField.FieldLogicalName}`).parent().removeClass('has-error');
            $(`input#${validation.AdministrativeAreaField.FieldLogicalName}`).parent().removeClass('has-error');
            $(`input#${validation.PostalCodeField.FieldLogicalName}`).parent().removeClass('has-error');
            if (validation.RegionCodeField) {
                $(`input#${validation.RegionCodeField.FieldLogicalName}`).parent().removeClass('has-error');
            }
        };

        var _openAddressConfirmationDialog = (validation, request, response) => new Promise((resolve, reject) => {
            var originalAddressFormatted = _formatAddressLines(request);
            $('#original-address-formatted').html(originalAddressFormatted);

            var suggestedAddressFormatted = _formatAddressLines(response.Address, request);
            $('#suggested-address-formatted').html(suggestedAddressFormatted);

            _setupAddressModalInput("addressmodal-old-street1",
                validation.AddressLineFields?.[0], request.AddressLines?.[0]);
            _setupAddressModalInput("addressmodal-old-street2",
                validation.AddressLineFields?.[1], request.AddressLines?.[1]);
            _setupAddressModalInput("addressmodal-old-city", validation.LocalityField, request.Locality);
            _setupAddressModalInput("addressmodal-old-state", validation.AdministrativeAreaField, request.AdministrativeArea);
            _setupAddressModalInput("addressmodal-old-zip", validation.PostalCodeField, request.PostalCode);
            _setupAddressModalInput("addressmodal-old-country", validation.RegionCodeField, request.RegionCode);

            _setupAddressModalInput("addressmodal-street1",
                validation.AddressLineFields?.[0], response.Address.AddressLines?.[0]);
            _setupAddressModalInput("addressmodal-street2",
                validation.AddressLineFields?.[1], response.Address.AddressLines?.[1]);
            _setupAddressModalInput("addressmodal-city", validation.LocalityField, response.Address.Locality);
            _setupAddressModalInput("addressmodal-state", validation.AdministrativeAreaField, response.Address.AdministrativeArea);
            _setupAddressModalInput("addressmodal-zip", validation.PostalCodeField, response.Address.PostalCode);
            _setupAddressModalInput("addressmodal-country", validation.RegionCodeField, response.Address.RegionCode);
            
            const closeButton = $("button#addressModalCancelBtn")[0];
            closeButton.onclick = () => {
                closeButton.onclick = null;
                reject();
            }

            const noChangesButton = $("button#addressModalNoChangesBtn")[0];
            noChangesButton.onclick = () => {
                $("#addressModal").modal("hide");
                noChangesButton.onclick = null;
                resolve();
            }

            const acceptButton = $("button#addressModalSaveBtn")[0];
            acceptButton.onclick = () => {
                $("#addressModal").modal("hide");
                acceptButton.onclick = null;
                resolve(response.Address);
            }

            $("#addressModal").modal();
        });

        var _formatAddressLines = function(address, comparisonAddress = null) {
            const street1 = _formatAddressLine(address.AddressLines?.[0], comparisonAddress?.AddressLines?.[0]);
            const street2 = _formatAddressLine(address.AddressLines?.[1], comparisonAddress?.AddressLines?.[1]);
            const city = _formatAddressLine(address.Locality, comparisonAddress?.Locality);
            const stateCode = _formatAddressLine(address.AdministrativeArea, comparisonAddress?.AdministrativeArea);
            const postalCode = _formatAddressLine(address.PostalCode, comparisonAddress?.PostalCode);

            return `${street1}</br>${street2}${street2 ? '</br>' : ''}${city}, ${stateCode} ${postalCode}`;
        };

        var _formatAddressLine = function(addressLine, comparisonLine) {
            if (!addressLine) {
                return '';
            }

            if (!comparisonLine) {
                return addressLine;
            }
          
            const isDifferent = comparisonLine?.toLowerCase() !== addressLine?.toLowerCase();

            return isDifferent
                ? `<em class="text-danger text-weight-semibold">${addressLine}</em>`
                : addressLine;
        };

        var _setupAddressModalInput = function (inputId, validationField, value)
        {
            const input = $(`input#${inputId}`);
            if (validationField) {
                input.val(value);
                input.labels().text(validationField.Label);
                input.show();
                input.labels().show();
            }
            else {
                input.hide();
                input.labels().hide();
            }
        }

        var _getEygaApiContextFromForm = function () {
            var regardingId = $("#hdn-recordid").val();
            var regardingType = $("#hdn-recordtype").val();
            var userId = $("#hdn-userid").val();
            return new FRCC.Portal.Utilities.EygaApiContext(regardingId, regardingType, userId)
        }

        var _setupFieldValidation = function(validation) {
            switch(validation.FieldValidationType) {
                case 643_260_000: // Equals
                    _addPageValidator(validation, function() {
                        var pageValue = $(`input#${validation.Identifier}`)[0].value;
                        var comparisonValue = validation.Value;
                        if (pageValue == comparisonValue) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    });
                    break;
                case 643_260_001: // Less Than
                    _addPageValidator(validation, function() {
                        var pageValue = $(`input#${validation.Identifier}`)[0].value;
                        var comparisonValue = validation.Value;
                        if (!isNaN(pageValue) && Number(pageValue) < Number(comparisonValue)) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    });
                    break;
                case 643_260_006: // Valid Address
                    _addPageValidator(validation, function () {
                        var response = FRCC.Portal.Validations.PreValidationResults.get(validation.Identifier);
                        return response === true;
                    });
                    break;
            };
        };

        var _setupTableValidation = function(validation) {
            switch(validation.TableValidationType) {
                case 643_260_000: // At Least One Record
                    _addPageValidator(validation, function() {
                        var tableRow = $(`#${validation.Identifier} table tbody tr`);
                        return tableRow.length > 0;
                    });
                    break;
            };
        };

        var _addPageValidator = function(validation, validationFunction) {
            if (typeof(Page_Validators) == 'undefined') return;
            var validator = document.createElement('span');
            validator.style.display = "none";
            validator.id = `validator-${validation.Identifier}`;
            validator.controltovalidate = "";
            var href = (validation.AddressLineFields && validation.AddressLineFields.length) > 0 ? 
                validation.AddressLineFields[0].FieldLogicalName : 
                (validation.Identifier !== null ? validation.Identifier : validation.StepNumber);
            validator.errormessage = `<a href='#${href}'>${validation.Message}</a>`;
			/* Check for a hard-coded "ValidationGroup" for places where are using validation outside of
			 * applications. Otherwise, in the form builder, we use a patter on of validation{step number} */
			validator.validationGroup = validation.ValidationGroup !== undefined && validation.ValidationGroup !== null
				? validation.ValidationGroup
				: `validation-${validation.StepNumber}`;
            validator.initialvalue = "";
            validator.evaluationfunction = validationFunction;
            Page_Validators.push(validator);
        };

        return {
            setupValidations: setupValidations,

            PreValidationResults: new PreValidationResults()
        }
    })();

})(this);